/*
 * CCU4.c
 *
 *  Created on: 30 Jul, 2015
 *      Author: toochang
 */
#include "XMC12_INIT.h"

//Project Macro definitions:
/* CCU4 Macros */

// Frequency CCU4 = 64MHz
#define PWM_PERIOD		6400	//10KHz
//#define PWM_PERIOD		3200	//20KHz

#define SLICE0_NUMBER    		(0U)

//XMC Capture/Compare Unit 4 (CCU4) Configuration:

XMC_CCU4_SLICE_COMPARE_CONFIG_t SLICE0_config =
{
  .timer_mode 		= (uint32_t) XMC_CCU4_SLICE_TIMER_COUNT_MODE_EA,
  .monoshot 		= (uint32_t) false,
  .shadow_xfer_clear 	= (uint32_t) 0,
  .dither_timer_period 	= (uint32_t) 0,
  .dither_duty_cycle 	= (uint32_t) 0,
  .prescaler_mode 	= (uint32_t) XMC_CCU4_SLICE_PRESCALER_MODE_NORMAL,
  .mcm_enable 		= (uint32_t) 0,
  .prescaler_initval   	= (uint32_t) 0,	//set prescaler to 2^9
  .float_limit		= (uint32_t) 0,
  .dither_limit		= (uint32_t) 0,
  .passive_level 	= (uint32_t) XMC_CCU4_SLICE_OUTPUT_PASSIVE_LEVEL_LOW,
  .timer_concatenation	= (uint32_t) 0
};

XMC_GPIO_CONFIG_t SLICE0_OUTPUT_config =
{
   .mode = XMC_GPIO_MODE_OUTPUT_PUSH_PULL_ALT4,
   .input_hysteresis = XMC_GPIO_INPUT_HYSTERESIS_STANDARD,
   .output_level = XMC_GPIO_OUTPUT_LEVEL_LOW,
};

void CCU4_Init (void)
{

	/* Enable clock, enable prescaler block and configure global control */
	XMC_CCU4_Init(CCU40, XMC_CCU4_SLICE_MCMS_ACTION_TRANSFER_PR_CR);

	/* Start the prescaler and restore clocks to slices */
	XMC_CCU4_StartPrescaler(CCU40);

	/* Start of CCU4 configurations */
	/* Ensure fCCU reaches CCU40 */
	XMC_CCU4_SetModuleClock(CCU40, XMC_CCU4_CLOCK_SCU);

	//Configure Slice(s) Functions, Interrupts and Start-up
	/* Initialize the Slice */
	XMC_CCU4_SLICE_CompareInit(CCU40_CC40, &SLICE0_config);

	/* Frequency CCU4 = 64MHz */
	XMC_CCU4_SLICE_SetTimerCompareMatch(CCU40_CC40, PWM_PERIOD/2);
	XMC_CCU4_SLICE_SetTimerPeriodMatch(CCU40_CC40, PWM_PERIOD);

	/* Enable shadow transfer for PWM Slice */
	XMC_CCU4_EnableShadowTransfer(CCU40, \
		(uint32_t)(XMC_CCU4_SHADOW_TRANSFER_SLICE_0|XMC_CCU4_SHADOW_TRANSFER_PRESCALER_SLICE_0));

	/* Enable events */
	XMC_CCU4_SLICE_EnableEvent(CCU40_CC40, XMC_CCU4_SLICE_IRQ_ID_PERIOD_MATCH);

	/* Connect event to SR2 */
	XMC_CCU4_SLICE_SetInterruptNode(CCU40_CC40, XMC_CCU4_SLICE_IRQ_ID_PERIOD_MATCH, XMC_CCU4_SLICE_SR_ID_2);

	/* Set NVIC priority */
	NVIC_SetPriority(CCU40_2_IRQn, 3U);
	NVIC_EnableIRQ(CCU40_2_IRQn);

	/* Enable CCU4 PWM output */
	XMC_GPIO_Init(P0_0, &SLICE0_OUTPUT_config);

	/* Get the slice out of idle mode */
	XMC_CCU4_EnableClock(CCU40, SLICE0_NUMBER);

	/* Start the TImer*/
	XMC_CCU4_SLICE_StartTimer(CCU40_CC40);
}

//The CCU interrupt handler function is a period match interrupt handler.
void CCU40_2_IRQHandler(void)
{
	XMC_CCU4_SLICE_ClearEvent(CCU40_CC40, XMC_CCU4_SLICE_IRQ_ID_PERIOD_MATCH);

}
